package cz.cvut.fel.pr1.cars;

import cz.cvut.fel.pr1.cars.model.Car;
import cz.cvut.fel.pr1.data.CarLinkedList;
import cz.cvut.fel.pr1.data.RaceResult;
import cz.cvut.fel.pr1.data.TimeSpan;
import cz.cvut.fel.pr1.data.utils.ArrayUtil;

/**
 * The Race class represents car race. It allows to add vehicles until the race is started,
 * afterwards the instance provides results of the particular race.
 * @author balikm1
 */
public class Race {
    private final int length;
    private final CarLinkedList startPositions;
    private boolean raceOpen;
    private RaceResult[] raceResults;
    
    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated Race object that represents the car race
     * @param length length of the race track
     */
    public Race(int length) {
        this.length = length;
        startPositions = new CarLinkedList();
        raceOpen = true;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="public methods">
    /**
     * Appends the specified car to the last starting position.
     * @param car the car that takes part in the race
     * @return true if the car was added sucessfuly
     */
    public boolean addRacingCar(Car car) {
        if (!raceOpen) return false;
        startPositions.add(car);
        return true;
    }
    
    /**
     * Get the winner of the race.
     * @return the wining car
     */
    public Car getWinner() {
        //  note the cached race results
        if (raceResults == null) {
            raceResults = getRaceResults();
        }
        return raceResults[0].getKey();
    }
    
    /**
     * Get the racing time of the winner
     * @return time interval specifying total racing time of the winner
     */
    public TimeSpan getWinningTime() {
        //  note the cached race results
        if (raceResults == null) {
            raceResults = getRaceResults();
        }
        return raceResults[0].getValue();
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="helper methods">
    private RaceResult[] getRaceResults() {
        raceOpen = false;
        Car[] startPositionsArray = this.startPositions.toArray();
        RaceResult[] results = new RaceResult[startPositionsArray.length];
        for (int i=0; i<startPositionsArray.length; i++) {
            int seconds = length / startPositionsArray[i].getSpeed();
            results[i] = new RaceResult(startPositionsArray[i], new TimeSpan(seconds));
        }
        ArrayUtil.sort(results);
        return results;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public String toString() {
        if (raceOpen) {
            return startPositions.toString();
        } else {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < raceResults.length; i++) {
                sb.append(String.format("%s, time: %s.\n", raceResults[i].getKey(), raceResults[i].getValue()));
            }
            return sb.toString();
        }
    }
    //</editor-fold>
}
