package cz.cvut.fel.pr1.cars.model;

import java.util.Objects;
import java.util.UUID;

/**
 * The Car class represents vehicles of various brands and describes some important car's properties.
 * @author balikm1
 */
public class Car {
    private static int numberOfExistingCars = 0;
    private final String manufacturer;
    private final String modelName;
    private final int year;
    private int speed;
    private final Engine engine;
    private final UUID vinCode;
    private ServiceBook serviceBook;
    
    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated Car object that represents a vehicle with particular properties.
     * @param manufacturer the car manufacturer
     * @param modelName name of the car model
     * @param year year of manufacture
     * @param engineType type of car's engine
     * @param speed the speed the car is able to drive in a race
     */
    public Car(String manufacturer, String modelName, int year, String engineType, int speed) {
        this.manufacturer = manufacturer;
        this.modelName = modelName;
        this.year = year;
        this.engine = new Engine(engineType);
        this.speed = speed;
        this.vinCode = UUID.randomUUID();
        
        numberOfExistingCars++;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="properties">
    /**
     * Get the car manufacturer
     * @return the car manufacturer
     */
    public String getManufacturer() {
        return manufacturer;
    }

    /**
     * Get name of the car model
     * @return name of the car model
     */
    public String getModelName() {
        return modelName;
    }

    /**
     * Get year of manufacture
     * @return year of manufacture
     */
    public int getYear() {
        return year;
    }

    /**
     * Get the speed the car is able to drive in a race
     * @return the speed the car is able to drive in a race
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * Set the speed the car is able to drive in a race
     * @param speed the speed the car is able to drive in a race
     */
    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /**
     * Get associated Engine object representing the engine of the car
     * @return associated Engine object representing the engine of the car
     */
    public Engine getEngine() {
        return engine;
    }

    /**
     * Get associated UUID object representing a unique identifier of the car
     * @return associated UUID object representing a unique identifier of the car
     */
    public UUID getVinCode() {
        return vinCode;
    }

    /**
     * Get associated ServiceBook object representing a book of car's service records
     * @return associated ServiceBook object representing a book of car's service records
     */
    public ServiceBook getServiceBook() {
        return serviceBook;
    }
    
    /**
     * Set a reference to the car's book of service records
     * @param serviceBook book of car's service records
     */
    void setServiceBook(ServiceBook serviceBook) {
        this.serviceBook = serviceBook;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 79 * hash + Objects.hashCode(this.vinCode);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Car other = (Car) obj;
        return Objects.equals(this.vinCode, other.vinCode);
    }

    @Override
    public String toString() {
        return String.format("%s %s year %s VIN: %s", manufacturer, modelName, year, vinCode);
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="static methods">
    /**
     * Get number of manufactured cars
     * @return number of manufactured cars
     */
    public static int getNumberOfExistingCars() {
        return numberOfExistingCars;
    }
    //</editor-fold>
}
