package cz.cvut.fel.pr1.cars.model;

import java.util.Arrays;
import java.util.Objects;

/**
 * The ServiceBook class represents a book of car's service records.
 * @author balikm1
 */
public class ServiceBook {
    private static final int SERVICE_RECORDS_COUNT = 10;
    private final Car car;
    private final String[] serviceRecords;
    private int freeRecord;

    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated ServiceBook object that represents a book of car's service records.
     * The service book is assigned to the corresponding car using a bidirectional relationship.
     * <p>
     * Note: Leaking this in constructor is only 'safe' in a single-threaded setting. See http://docs.oracle.com/javase/specs/jls/se7/html/jls-17.html for details.
     * </p>
     * @param car the car whose service records the book keeps
     */
    @SuppressWarnings("LeakingThisInConstructor")
    public ServiceBook(Car car) {
        this.car = car;
        car.setServiceBook(this);
        serviceRecords = new String[SERVICE_RECORDS_COUNT];
        freeRecord = 0;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="properties">
    /**
     * Get car in evidence of this service book
     * @return car in evidence of this service book
     */
    public Car getCar() {
        return car;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="public methods">
    /**
     * Adds the specified service record into the service book
     * @param record service record to add
     * @return true if service record was added successfuly, false if there is no more space to add new record
     */
    public boolean addRecord(String record) {
        if (freeRecord >= serviceRecords.length)
            return false;
        serviceRecords[freeRecord++] = record;
        return true;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + Objects.hashCode(this.car);
        hash = 97 * hash + Arrays.deepHashCode(this.serviceRecords);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ServiceBook other = (ServiceBook) obj;
        if (!Objects.equals(this.car, other.car)) {
            return false;
        }
        if (!Arrays.deepEquals(this.serviceRecords, other.serviceRecords)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < freeRecord; i++) {
            if (serviceRecords[i] != null && !serviceRecords[i].isEmpty()) {
                sb.append(serviceRecords[i]).append("\n");
            }
        }
        return sb.toString();
    }
    //</editor-fold>
}
